@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the Qt Designer of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "iconselector_p.h"
 #include "qdesigner_utils_p.h"
@@ -42,7 +17,6 @@
 
 #include <QtWidgets/qtoolbutton.h>
 #include <QtWidgets/qcombobox.h>
-#include <QtWidgets/qaction.h>
 #include <QtWidgets/qdialogbuttonbox.h>
 #include <QtWidgets/qpushbutton.h>
 #include <QtWidgets/qdialog.h>
@@ -53,9 +27,13 @@
 #include <QtWidgets/qdialogbuttonbox.h>
 #include <QtWidgets/qlineedit.h>
 #include <QtWidgets/qlabel.h>
-#include <QtGui/qvalidator.h>
-#include <QtCore/qdebug.h>
 
+#include <QtGui/qaction.h>
+#include <QtGui/qvalidator.h>
+
+#include <QtCore/qdebug.h>
+#include <QtCore/qlist.h>
+#include <QtCore/qstringlistmodel.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -85,7 +63,7 @@ private:
 };
 
 LanguageResourceDialogPrivate::LanguageResourceDialogPrivate(QDesignerResourceBrowserInterface *rb) :
-    q_ptr(0),
+    q_ptr(nullptr),
     m_browser(rb),
     m_dialogButtonBox(new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel))
 {
@@ -158,11 +136,11 @@ QString LanguageResourceDialog::currentPath() const
 LanguageResourceDialog* LanguageResourceDialog::create(QDesignerFormEditorInterface *core, QWidget *parent)
 {
     if (QDesignerLanguageExtension *lang = qt_extension<QDesignerLanguageExtension *>(core->extensionManager(), core))
-        if (QDesignerResourceBrowserInterface *rb = lang->createResourceBrowser(0))
+        if (QDesignerResourceBrowserInterface *rb = lang->createResourceBrowser(nullptr))
             return new LanguageResourceDialog(rb, parent);
-    if (QDesignerResourceBrowserInterface *rb = core->integration()->createResourceBrowser(0))
+    if (QDesignerResourceBrowserInterface *rb = core->integration()->createResourceBrowser(nullptr))
         return new LanguageResourceDialog(rb, parent);
-    return 0;
+    return nullptr;
 }
 
 // ------------ IconSelectorPrivate
@@ -176,10 +154,10 @@ static inline QPixmap emptyPixmap()
 
 class IconSelectorPrivate
 {
-    IconSelector *q_ptr;
+    IconSelector *q_ptr = nullptr;
     Q_DECLARE_PUBLIC(IconSelector)
 public:
-    IconSelectorPrivate();
+    IconSelectorPrivate() = default;
 
     void slotStateActivated();
     void slotSetActivated();
@@ -195,30 +173,17 @@ public:
     QMap<int, QPair<QIcon::Mode, QIcon::State> > m_indexToState;
 
     const QIcon m_emptyIcon;
-    QComboBox *m_stateComboBox;
-    QToolButton *m_iconButton;
-    QAction *m_resetAction;
-    QAction *m_resetAllAction;
+    QComboBox *m_stateComboBox = nullptr;
+    QToolButton *m_iconButton = nullptr;
+    QAction *m_resetAction = nullptr;
+    QAction *m_resetAllAction = nullptr;
     PropertySheetIconValue m_icon;
-    DesignerIconCache *m_iconCache;
-    DesignerPixmapCache *m_pixmapCache;
-    QtResourceModel *m_resourceModel;
-    QDesignerFormEditorInterface *m_core;
+    DesignerIconCache *m_iconCache = nullptr;
+    DesignerPixmapCache *m_pixmapCache = nullptr;
+    QtResourceModel *m_resourceModel = nullptr;
+    QDesignerFormEditorInterface *m_core = nullptr;
 };
 
-IconSelectorPrivate::IconSelectorPrivate() :
-    q_ptr(0),
-    m_emptyIcon(emptyPixmap()),
-    m_stateComboBox(0),
-    m_iconButton(0),
-    m_resetAction(0),
-    m_resetAllAction(0),
-    m_iconCache(0),
-    m_pixmapCache(0),
-    m_resourceModel(0),
-    m_core(0)
-{
-}
 void IconSelectorPrivate::slotUpdate()
 {
     QIcon icon;
@@ -342,10 +307,10 @@ bool IconSelector::checkPixmap(const QString &fileName, CheckMode cm, QString *e
 static QString imageFilter()
 {
     QString filter = QApplication::translate("IconSelector", "All Pixmaps (");
-    const QList<QByteArray> supportedImageFormats = QImageReader::supportedImageFormats();
+    const auto supportedImageFormats = QImageReader::supportedImageFormats();
     const QString jpeg = QStringLiteral("JPEG");
-    const int count = supportedImageFormats.count();
-    for (int i = 0; i< count; ++i) {
+    const qsizetype count = supportedImageFormats.size();
+    for (qsizetype i = 0; i < count; ++i) {
         if (i)
             filter += QLatin1Char(' ');
         filter += QStringLiteral("*.");
@@ -521,35 +486,34 @@ void IconSelector::setPixmapCache(DesignerPixmapCache *pixmapCache)
 
 // --- IconThemeEditor
 
-// Validator for theme line edit, accepts empty or non-blank strings.
-class BlankSuppressingValidator : public QValidator {
-public:
-    explicit BlankSuppressingValidator(QObject * parent = 0) : QValidator(parent) {}
-
-    State validate(QString &input, int &pos) const override
-    {
-        const int blankPos = input.indexOf(QLatin1Char(' '));
-        if (blankPos != -1) {
-            pos = blankPos;
-            return Invalid;
-        }
-        return Acceptable;
-    }
-};
-
 struct IconThemeEditorPrivate {
     IconThemeEditorPrivate();
 
     const QPixmap m_emptyPixmap;
-    QLineEdit *m_themeLineEdit;
+    QComboBox *m_themeComboBox;
     QLabel *m_themeLabel;
+    QStringListModel *m_themeModel;
 };
 
 IconThemeEditorPrivate::IconThemeEditorPrivate() :
     m_emptyPixmap(emptyPixmap()),
-    m_themeLineEdit(new QLineEdit),
-    m_themeLabel(new QLabel)
+    m_themeComboBox(new QComboBox),
+    m_themeLabel(new QLabel),
+    m_themeModel(new QStringListModel)
 {
+    QStringList iconNames;
+    QFile file(QStringLiteral(":/qt-project.org/designer/icon-naming-spec.txt"));
+    if (file.open(QIODevice::ReadOnly)) {
+        while (!file.atEnd()) {
+            const auto line = file.readLine().trimmed();
+            if (line.isEmpty() || line.startsWith('#'))
+                continue;
+            iconNames.append(QString::fromUtf8(line));
+        }
+        file.close();
+    }
+    std::sort(iconNames.begin(), iconNames.end());
+    m_themeModel->setStringList(iconNames);
 }
 
 IconThemeEditor::IconThemeEditor(QWidget *parent, bool wantResetButton) :
@@ -569,11 +533,11 @@ IconThemeEditor::IconThemeEditor(QWidget *parent, bool wantResetButton) :
     themeLabelVLayout->addSpacerItem(new QSpacerItem(0, 0, QSizePolicy::Ignored, QSizePolicy::MinimumExpanding));
     mainHLayout->addLayout(themeLabelVLayout);
 
-    d->m_themeLineEdit = new QLineEdit;
-    d->m_themeLineEdit->setValidator(new BlankSuppressingValidator(d->m_themeLineEdit));
-    connect(d->m_themeLineEdit, &QLineEdit::textChanged, this, &IconThemeEditor::slotChanged);
-    connect(d->m_themeLineEdit, &QLineEdit::textEdited, this, &IconThemeEditor::edited);
-    mainHLayout->addWidget(d->m_themeLineEdit);
+    d->m_themeComboBox->setModel(d->m_themeModel);
+    d->m_themeComboBox->setCurrentIndex(-1);
+    connect(d->m_themeComboBox, &QComboBox::currentTextChanged, this, &IconThemeEditor::slotChanged);
+    connect(d->m_themeComboBox, &QComboBox::currentTextChanged, this, &IconThemeEditor::edited);
+    mainHLayout->addWidget(d->m_themeComboBox);
 
     if (wantResetButton) {
         QToolButton *themeResetButton = new QToolButton;
@@ -583,14 +547,14 @@ IconThemeEditor::IconThemeEditor(QWidget *parent, bool wantResetButton) :
     }
 
     setLayout(mainHLayout);
-    setFocusProxy(d->m_themeLineEdit);
+    setFocusProxy(d->m_themeComboBox);
 }
 
 IconThemeEditor::~IconThemeEditor() = default;
 
 void IconThemeEditor::reset()
 {
-    d->m_themeLineEdit->clear();
+    d->m_themeComboBox->setCurrentIndex(-1);
     emit edited(QString());
 }
 
@@ -603,8 +567,7 @@ void IconThemeEditor::updatePreview(const QString &t)
 {
     // Update preview label with icon.
     if (t.isEmpty() || !QIcon::hasThemeIcon(t)) { // Empty
-        const QPixmap *currentPixmap = d->m_themeLabel->pixmap();
-        if (currentPixmap == 0 || currentPixmap->cacheKey() != d->m_emptyPixmap.cacheKey())
+        if (d->m_themeLabel->pixmap(Qt::ReturnByValue).cacheKey() != d->m_emptyPixmap.cacheKey())
             d->m_themeLabel->setPixmap(d->m_emptyPixmap);
     } else {
         const QIcon icon = QIcon::fromTheme(t);
@@ -614,12 +577,12 @@ void IconThemeEditor::updatePreview(const QString &t)
 
 QString IconThemeEditor::theme() const
 {
-    return d->m_themeLineEdit->text();
+    return d->m_themeComboBox->currentText();
 }
 
 void IconThemeEditor::setTheme(const QString &t)
 {
-    d->m_themeLineEdit->setText(t);
+    d->m_themeComboBox->setCurrentText(t);
 }
 
 } // qdesigner_internal
